@@ -1,4 +1,17 @@
-"""Aerleon "Generate" API
+""" Aerleon API
+
+This module accepts plain Python dictionaries and lists as input.
+It allows users to use Aerleon's functionality without having to
+deal with YAML files, command line scripts, or config files.
+
+This module exposes two APIs:
+
+* Generate API: Provides the policy-to-ACL capabilities of the aclgen
+  command line tool.
+* Check API: Provides the policy query capabilities of the aclcheck
+  command line tool.
+
+## Aerleon "Generate" API
 
 The Generate API provides the full policy-to-ACL capabilities of
 the aclgen command line tool. It accepts as input plain Python
@@ -170,6 +183,14 @@ ip access-list extended test-filter
 
 exit
 ```
+
+
+## Aerleon "AclCheck" API
+
+The AclCheck API provides the policy query capabilities of the aclcheck
+command line tool. It accepts as input plain Python dictionaries and lists.
+
+
 """
 import copy
 import multiprocessing
@@ -186,16 +207,23 @@ from aerleon.aclgen import (
     WriteFiles,
     WriteList,
 )
-from aerleon.lib import aclgenerator, naming, plugin_supervisor, policy, policy_builder
+from aerleon.lib import (
+    aclcheck,
+    aclgenerator,
+    naming,
+    plugin_supervisor,
+    policy,
+    policy_builder,
+)
 
 
 def Generate(
-    policies,
-    definitions,
+    policies: "list[policy_builder.PolicyDict]",
+    definitions: naming.Naming,
     output_directory: pathlib.Path = None,
-    optimize=False,
-    shade_check=False,
-    expiration_weeks=2,
+    optimize: bool = False,
+    shade_check: bool = False,
+    expiration_weeks: int = 2,
 ) -> "dict[str, str]":
     """Generate ACLs from policies.
 
@@ -243,14 +271,14 @@ def Generate(
 
 
 def _Generate(
-    policies,
+    policies: "list[policy_builder.PolicyDict]",
     definitions: naming.Naming,
     context: multiprocessing.context.BaseContext,
-    output_directory=None,
-    optimize=False,
-    shade_check=False,
-    exp_info=2,
-    max_renderers=1,
+    output_directory: pathlib.Path = None,
+    optimize: bool = False,
+    shade_check: bool = False,
+    exp_info: int = 2,
+    max_renderers: int = 1,
 ) -> "dict[str, str]":
 
     # thead-safe list for storing files to write
@@ -306,38 +334,19 @@ def _Generate(
 
 
 def _GenerateACL(
-    input_policy: policy.Policy,
+    input_policy: policy_builder.PolicyDict,
     definitions: naming.Naming,
     write_files: WriteList,
     generated_configs: dict,
     output_directory: pathlib.Path = None,
-    optimize=False,
-    shade_check=False,
-    exp_info=2,
+    optimize: bool = False,
+    shade_check: bool = False,
+    exp_info: int = 2,
 ):
-    raw_filters = []
-    input_filters = input_policy["filters"]
     filename = input_policy.get("filename")
-    for filter in input_filters:
-
-        filter_header = filter["header"]
-        header_targets = filter_header["targets"]
-        raw_filter_header = policy_builder.RawFilterHeader(
-            targets=header_targets, kvs=filter_header
-        )
-
-        raw_terms = []
-        filter_terms = filter["terms"]
-        for term in filter_terms:
-            raw_term = policy_builder.RawTerm(name=term["name"], kvs=term)
-            raw_terms.append(raw_term)
-
-        raw_filters.append(policy_builder.RawFilter(header=raw_filter_header, terms=raw_terms))
-
-    raw_policy = policy_builder.RawPolicy(filename=filename, filters=raw_filters)
     try:
         policy_obj = policy.FromBuilder(
-            policy_builder.PolicyBuilder(raw_policy, definitions, optimize, shade_check)
+            policy_builder.PolicyBuilder(input_policy, definitions, optimize, shade_check)
         )
     except policy.ShadingError as e:
         logging.warning('shading errors for %s:\n%s', filename, e)
@@ -395,3 +404,24 @@ def _GenerateACL(
             raise ACLGeneratorError(
                 'Error generating target ACL for %s:\n%s' % (filename, e)
             ) from e
+
+
+def AclCheck(
+    input_policy: policy_builder.PolicyDict,
+    definitions: naming.Naming,
+    src: str = None,
+    dst: str = None,
+    sport: str = None,
+    dport: str = None,
+    proto: str = None,
+):
+    filename = input_policy.get("filename")
+    try:
+        check = aclcheck.AclCheck.FromPolicyDict(
+            input_policy, definitions, src, dst, sport, dport, proto
+        )
+        return check.Summarize()
+    except (policy.Error, naming.Error) as e:
+        raise ACLParserError(
+            'Error parsing policy %s:\n%s%s' % (filename, sys.exc_info()[0], sys.exc_info()[1])
+        ) from e
