@@ -17,11 +17,14 @@
 """Cisco generator."""
 
 import ipaddress
-from typing import Union, cast
+from typing import Any, Dict, List, Optional, Set, Tuple, Union, cast
 
 from absl import logging
 
 from aerleon.lib import aclgenerator, addressbook, nacaddr, summarizer
+from aerleon.lib.nacaddr import IPv4, IPv6
+from aerleon.lib.policy import Policy, Term
+from aerleon.lib.summarizer import DSMNet
 
 _ACTION_TABLE = {
     'accept': 'permit',
@@ -62,7 +65,9 @@ class ExtendedACLTermError(Error):
 class TermStandard:
     """A single standard ACL Term."""
 
-    def __init__(self, term, filter_name, platform='cisco', verbose=True):
+    def __init__(
+        self, term: Term, filter_name: str, platform: str = 'cisco', verbose: bool = True
+    ) -> None:
         self.term = term
         self.filter_name = filter_name
         self.platform = platform
@@ -105,7 +110,7 @@ class TermStandard:
             )
             self.dscpstring = ' dscp' + self.term.dscp_match
 
-    def __str__(self):
+    def __str__(self) -> str:
         ret_str = []
 
         # Term verbatim output - this will skip over normal term creation
@@ -117,84 +122,38 @@ class TermStandard:
             return '\n'.join(ret_str)
 
         v4_addresses = [x for x in self.term.address if not isinstance(x, nacaddr.IPv6)]
+        prefix = ''
         if self.filter_name.isdigit():
-            if self.verbose:
-                ret_str.append('access-list %s remark %s' % (self.filter_name, self.term.name))
-                comments = aclgenerator.WrapWords(self.term.comment, _COMMENT_MAX_WIDTH)
-                for comment in comments:
-                    ret_str.append('access-list %s remark %s' % (self.filter_name, comment))
+            prefix = f'access-list {self.filter_name}'
 
-            action = _ACTION_TABLE.get(str(self.term.action[0]))
-            if v4_addresses:
-                for addr in v4_addresses:
+        if self.verbose:
+            ret_str.append(f'{prefix} remark {self.term.name}')
+            comments = aclgenerator.WrapWords(self.term.comment, _COMMENT_MAX_WIDTH)
+            for comment in comments:
+                ret_str.append(f'{prefix} remark {comment}')
+
+        action = _ACTION_TABLE.get(str(self.term.action[0]))
+        if v4_addresses:
+            for addr in v4_addresses:
+                if self.platform == 'arista':
                     if addr.prefixlen == 32:
                         ret_str.append(
-                            'access-list %s %s %s%s%s'
-                            % (
-                                self.filter_name,
-                                action,
-                                addr.network_address,
-                                self.logstring,
-                                self.dscpstring,
-                            )
+                            f'{prefix} {action} host {addr.network_address}{self.logstring}{self.dscpstring}'
                         )
                     else:
                         ret_str.append(
-                            'access-list %s %s %s %s%s%s'
-                            % (
-                                self.filter_name,
-                                action,
-                                addr.network_address,
-                                addr.hostmask,
-                                self.logstring,
-                                self.dscpstring,
-                            )
+                            f'{prefix} {action} {addr.network_address}/{addr.prefixlen}{self.logstring}{self.dscpstring}'
                         )
-            else:
-                ret_str.append(
-                    'access-list %s %s %s%s%s'
-                    % (self.filter_name, action, 'any', self.logstring, self.dscpstring)
-                )
+                elif addr.prefixlen == 32:
+                    ret_str.append(
+                        f'{prefix} {action} {addr.network_address}{self.logstring}{self.dscpstring}'
+                    )
+                else:
+                    ret_str.append(
+                        f'{prefix} {action} {addr.network_address} {addr.hostmask}{self.logstring}{self.dscpstring}'
+                    )
         else:
-            if self.verbose:
-                ret_str.append(' remark ' + self.term.name)
-                comments = aclgenerator.WrapWords(self.term.comment, _COMMENT_MAX_WIDTH)
-                if comments and comments[0]:
-                    for comment in comments:
-                        ret_str.append(' remark ' + str(comment))
-
-            action = _ACTION_TABLE.get(str(self.term.action[0]))
-            if v4_addresses:
-                for addr in v4_addresses:
-                    if addr.prefixlen == 32:
-                        ret_str.append(
-                            ' %s host %s%s%s'
-                            % (action, addr.network_address, self.logstring, self.dscpstring)
-                        )
-                    elif self.platform == 'arista':
-                        ret_str.append(
-                            ' %s %s/%s%s%s'
-                            % (
-                                action,
-                                addr.network_address,
-                                addr.prefixlen,
-                                self.logstring,
-                                self.dscpstring,
-                            )
-                        )
-                    else:
-                        ret_str.append(
-                            ' %s %s %s%s%s'
-                            % (
-                                action,
-                                addr.network_address,
-                                addr.hostmask,
-                                self.logstring,
-                                self.dscpstring,
-                            )
-                        )
-            else:
-                ret_str.append(' %s %s%s%s' % (action, 'any', self.logstring, self.dscpstring))
+            ret_str.append(f'{prefix} {action} any{self.logstring}{self.dscpstring}')
 
         return '\n'.join(ret_str)
 
@@ -220,22 +179,22 @@ class ObjectGroup:
       exit
     """
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.filter_name = ''
         self.addressbook = addressbook.Addressbook()
         self.terms = []
 
-    def AddTerm(self, term):
+    def AddTerm(self, term: Term) -> None:
         if term.source_address:
             self.addressbook.AddAddresses('', term.source_address)
         if term.destination_address:
             self.addressbook.AddAddresses('', term.destination_address)
         self.terms.append(term)
 
-    def AddName(self, filter_name):
+    def AddName(self, filter_name: str) -> None:
         self.filter_name = filter_name
 
-    def __str__(self):
+    def __str__(self) -> str:
         ret_str = ['\n']
         # netgroups will contain two-tuples of group name string and family int.
         netgroups = set()
@@ -438,7 +397,7 @@ class PortMap:
 
 
 class Term(aclgenerator.Term):
-    """A single ACL Term."""
+    """This is an general Term object that all other Cisco Terms should inherite from."""
 
     ALLOWED_PROTO_STRINGS = [
         'eigrp',
@@ -455,21 +414,19 @@ class Term(aclgenerator.Term):
         'sctp',
         'ahp',
     ]
-
     IPV4_ADDRESS = Union[nacaddr.IPv4, ipaddress.IPv4Network]
     IPV6_ADDRESS = Union[nacaddr.IPv6, ipaddress.IPv6Network]
 
     def __init__(
         self,
-        term,
-        af=4,
-        proto_int=True,
-        enable_dsmo=False,
-        term_remark=True,
-        platform='cisco',
-        verbose=True,
-    ):
-        super().__init__(term)
+        term: Term,
+        af: int = 4,
+        proto_int: bool = True,
+        enable_dsmo: bool = False,
+        term_remark: bool = True,
+        platform: str = 'cisco',
+        verbose: bool = True,
+    ) -> None:
         self.term = term
         self.proto_int = proto_int
         self.options = []
@@ -485,7 +442,7 @@ class Term(aclgenerator.Term):
         else:
             self.text_af = 'inet6'
 
-    def __str__(self):
+    def __str__(self) -> str:
         ret_str = ['\n']
 
         # Don't render icmpv6 protocol terms under inet, or icmp under inet6
@@ -501,14 +458,20 @@ class Term(aclgenerator.Term):
                 )
             )
             return ''
+
+        # verbose
         if self.verbose:
+            comments = []
             if self.term_remark:
-                ret_str.append(' remark ' + self.term.name)
+                comments.append(self.term.name)
             if self.term.owner:
-                self.term.comment.append('Owner: %s' % self.term.owner)
-            for comment in self.term.comment:
-                for line in comment.split('\n'):
-                    ret_str.append(' remark ' + str(line)[:100].rstrip())
+                comments.append(f'Owner: {self.term.owner}')
+            comments.extend(self.term.comment)
+
+            comments = aclgenerator.WrapWords(comments, _COMMENT_MAX_WIDTH)
+            if comments and comments[0]:
+                for comment in comments:
+                    ret_str.append(' remark %s' % str(comment))
 
         # Term verbatim output - this will skip over normal term creation
         # code by returning early.  Warnings provided in policy.py.
@@ -545,6 +508,7 @@ class Term(aclgenerator.Term):
             if 'ah' in protocol:
                 protocol = [x if x != 'ah' else '51' for x in protocol]
 
+        # addresses
         # source address
         if self.term.source_address:
             source_address = self.term.GetAddressOfVersion('source_address', self.af)
@@ -564,7 +528,8 @@ class Term(aclgenerator.Term):
                 source_address = summarizer.Summarize(source_address)
         else:
             # source address not set
-            source_address = ['any']
+            source_address = [nacaddr.IPv4('0.0.0.0/0', token='any')]
+        fixed_src_addresses = set([self._GetIpString(x) for x in source_address])
 
         # destination address
         if self.term.destination_address:
@@ -587,7 +552,21 @@ class Term(aclgenerator.Term):
                 destination_address = summarizer.Summarize(destination_address)
         else:
             # destination address not set
-            destination_address = ['any']
+            destination_address = [nacaddr.IPv4('0.0.0.0/0', token='any')]
+
+        fixed_dst_addresses = set([self._GetIpString(x) for x in destination_address])
+
+        # ports
+        source_port = [()]
+        destination_port = [()]
+
+        # source port
+        if self.term.source_port:
+            source_port = self._FixConsecutivePorts(self.term.source_port)
+
+        # destination port
+        if self.term.destination_port:
+            destination_port = self._FixConsecutivePorts(self.term.destination_port)
 
         # options
         opts = [str(x) for x in self.term.option]
@@ -597,7 +576,7 @@ class Term(aclgenerator.Term):
             if 'established' not in self.options:
                 self.options.append('established')
         # Using both 'fragments' and 'is-fragment', ref Github Issue #187
-        if ('ip' in protocol) and (('fragments' in opts) or ('is-fragment' in opts)):
+        if ('fragments' in opts) or ('is-fragment' in opts):
             if 'fragments' not in self.options:
                 self.options.append('fragments')
         # ACL-based Forwarding
@@ -626,18 +605,10 @@ class Term(aclgenerator.Term):
             self.options.append('nexthop1 %s %s' % (nexthop_protocol, nexthop))
             action = _ACTION_TABLE.get('accept')
 
+        # action
         if self.term.action:
             action = _ACTION_TABLE.get(str(self.term.action[0]))
 
-        # ports
-        source_port = [()]
-        destination_port = [()]
-        if self.term.source_port:
-            source_port = self._FixConsecutivePorts(self.term.source_port)
-
-        if self.term.destination_port:
-            destination_port = self._FixConsecutivePorts(self.term.destination_port)
-
         # logging
         if self.term.logging:
             self.options.append('log')
@@ -658,23 +629,24 @@ class Term(aclgenerator.Term):
         icmp_codes = ['']
         if self.term.icmp_code:
             icmp_codes = self.term.icmp_code
-        fixed_src_addresses = [self._GetIpString(x) for x in source_address]
-        fixed_dst_addresses = [self._GetIpString(x) for x in destination_address]
+
         fixed_opts = {}
         for p in protocol:
             fixed_opts[p] = self._FixOptions(p, self.options)
-        for saddr in fixed_src_addresses:
-            for daddr in fixed_dst_addresses:
-                for sport in source_port:
-                    for dport in destination_port:
-                        for proto in protocol:
+
+        # temlet constructor
+        for saddr in sorted(fixed_src_addresses):
+            for daddr in sorted(fixed_dst_addresses):
+                for sport in sorted(source_port):
+                    for dport in sorted(destination_port):
+                        for proto in sorted(protocol):
                             opts = fixed_opts[proto]
                             # cisconx uses icmp for both ipv4 and ipv6
                             if self.platform == 'cisconx':
                                 if self.af == 6:
                                     proto = 'icmp' if proto == 'icmpv6' else proto
-                            for icmp_type in icmp_types:
-                                for icmp_code in icmp_codes:
+                            for icmp_type in sorted(icmp_types):
+                                for icmp_code in sorted(icmp_codes):
                                     ret_str.extend(
                                         self._TermletToStr(
                                             action,
@@ -691,7 +663,7 @@ class Term(aclgenerator.Term):
 
         return '\n'.join(ret_str)
 
-    def _GetIpString(self, addr):
+    def _GetIpString(self, addr: Union[IPv6, IPv4, DSMNet]) -> str:
         """Formats the address object for printing in the ACL.
 
         Args:
@@ -699,6 +671,11 @@ class Term(aclgenerator.Term):
         Returns:
           An address string suitable for the ACL.
         """
+        # DSMO enabled
+        if isinstance(addr, summarizer.DSMNet):
+            return '%s %s' % summarizer.ToDottedQuad(addr, negate=True)
+        if addr.prefixlen == 0:
+            return 'any'
         if isinstance(addr, nacaddr.IPv4) or isinstance(addr, ipaddress.IPv4Network):
             addr = cast(self.IPV4_ADDRESS, addr)
             if addr.num_addresses > 1:
@@ -711,20 +688,17 @@ class Term(aclgenerator.Term):
             if addr.num_addresses > 1:
                 return addr.with_prefixlen
             return 'host %s' % (addr.network_address)
-        # DSMO enabled
-        if isinstance(addr, summarizer.DSMNet):
-            return '%s %s' % summarizer.ToDottedQuad(addr, negate=True)
         return addr
 
-    def _FormatPort(self, port, proto):
+    def _FormatPort(self, port: Union[Tuple[()], Tuple[int, int]], proto: Union[int, str]) -> str:
         """Returns a formatted port string for the range.
 
         Args:
-          port: str list or none, the port range.
-          proto: str representing proto (tcp, udp, etc).
+          port: The port range represented as a tuple..
+          proto: String or int representing a protocol.
 
         Returns:
-          A string suitable for the ACL.
+          A string used to filter a single or range of ports..
         """
         if not port:
             return ''
@@ -738,7 +712,9 @@ class Term(aclgenerator.Term):
             return 'range %s %s' % (port0, port1)
         return 'eq %s' % (port0)
 
-    def _FixOptions(self, proto, option):
+    def _FixOptions(
+        self, proto: Union[int, str], option: List[Union[str, Any]]
+    ) -> List[Union[str, Any]]:
         """Returns a set of options suitable for the given protocol.
 
         Fix done:
@@ -758,8 +734,17 @@ class Term(aclgenerator.Term):
         return sane_options
 
     def _TermletToStr(
-        self, action, proto, saddr, sport, daddr, dport, icmp_type, icmp_code, option
-    ):
+        self,
+        action: str,
+        proto: Union[int, str],
+        saddr: str,
+        sport: str,
+        daddr: str,
+        dport: str,
+        icmp_type: Union[int, str],
+        icmp_code: Union[int, str],
+        option: List[str],
+    ) -> List[str]:
         """Take the various compenents and turn them into a cisco acl line.
 
         Args:
@@ -771,7 +756,7 @@ class Term(aclgenerator.Term):
           dport: str, the destination port
           icmp_type: icmp-type numeric specification (if any)
           icmp_code: icmp-code numeric specification (if any)
-          option: list or none, optional, eg. 'logging' tokens.
+          option: list of options, eg. 'logging' tokens.
 
         Returns:
           string of the cisco acl line, suitable for printing.
@@ -796,7 +781,7 @@ class Term(aclgenerator.Term):
         non_empty_elements = [x for x in all_elements if x]
         return [' ' + ' '.join(non_empty_elements)]
 
-    def _FixConsecutivePorts(self, port_list):
+    def _FixConsecutivePorts(self, port_list: List[Tuple[int, int]]) -> List[Tuple[int, int]]:
         """Takes a list of tuples and expands the tuple if the range is two.
 
             http://www.cisco.com/warp/public/cc/pd/si/casi/ca6000/tech/65acl_wp.pdf
@@ -817,6 +802,10 @@ class Term(aclgenerator.Term):
         return temporary_port_list
 
 
+class ExtendedTerm(Term):
+    """A single ExtendedACL Term."""
+
+
 class ObjectGroupTerm(Term):
     """An individual term of an object-group'd acl.
 
@@ -832,100 +821,31 @@ class ObjectGroupTerm(Term):
     in the acl.
     """
 
-    # Protocols should be emitted as integers rather than strings.
-    _PROTO_INT = True
+    def _FormatPort(self, port: Union[Tuple[()], Tuple[int, int]], proto: str) -> str:
+        """Returns a formatted port string for the range.
 
-    def __init__(self, term, filter_name, platform='cisco', verbose=True):
-        super().__init__(term)
-        self.term = term
-        self.filter_name = filter_name
-        self.platform = platform
-        self.verbose = verbose
+        Args:
+          port: str list or none, the port range.
+          proto: str representing proto (tcp, udp, etc).
 
-    def __str__(self):
-        source_address_set = set()
-        destination_address_set = set()
-        ret_str = ['\n']
-        if self.verbose:
-            ret_str.append(' remark %s' % self.term.name)
-            comments = aclgenerator.WrapWords(self.term.comment, _COMMENT_MAX_WIDTH)
-            if comments and comments[0]:
-                for comment in comments:
-                    ret_str.append(' remark %s' % str(comment))
+        Returns:
+          A string suitable for the ACL.
+        """
+        if not port:
+            return ''
+        return f'port-group {port[0]}-{port[1]}'
 
-        # Term verbatim output - this will skip over normal term creation
-        # code by returning early.  Warnings provided in policy.py.
-        if self.term.verbatim:
-            for next_verbatim in self.term.verbatim:
-                if next_verbatim[0] == self.platform:
-                    ret_str.append(str(next_verbatim[1]))
-            return '\n'.join(ret_str)
+    def _GetIpString(self, addr: Union[IPv4, IPv6]) -> str:
+        """Formats the address object for printing in the ACL.
 
-        # protocol
-        if not self.term.protocol:
-            protocol = ['ip']
-
-        else:
-            protocol = [
-                proto
-                if proto in self.ALLOWED_PROTO_STRINGS or proto.isnumeric()
-                else self.PROTO_MAP.get(proto)
-                for proto in self.term.protocol
-            ]
-
-        # addresses
-        source_address = self.term.source_address
-        if not self.term.source_address:
-            source_address = [nacaddr.IPv4('0.0.0.0/0', token='any')]
-        source_address_set.add(source_address[0].parent_token)
-
-        destination_address = self.term.destination_address
-        if not self.term.destination_address:
-            destination_address = [nacaddr.IPv4('0.0.0.0/0', token='any')]
-        destination_address_set.add(destination_address[0].parent_token)
-        # ports
-        source_port = [()]
-        destination_port = [()]
-        if self.term.source_port:
-            source_port = self.term.source_port
-        if self.term.destination_port:
-            destination_port = self.term.destination_port
-        for saddr in source_address_set:
-            for daddr in destination_address_set:
-                for sport in source_port:
-                    for dport in destination_port:
-                        for proto in protocol:
-                            ret_str.append(
-                                self._TermletToStr(
-                                    _ACTION_TABLE.get(str(self.term.action[0])),
-                                    proto,
-                                    saddr,
-                                    sport,
-                                    daddr,
-                                    dport,
-                                )
-                            )
-
-        return '\n'.join(ret_str)
-
-    def _TermletToStr(self, action, proto, saddr, sport, daddr, dport):
-        """Output a portion of a cisco term/filter only, based on the 5-tuple."""
-        # Empty addr/port destinations should emit 'any'
-        if saddr and saddr != 'any':
-            saddr = 'net-group %s' % saddr
-        if daddr and daddr != 'any':
-            daddr = 'net-group %s' % daddr
-        # fix ports
-        if sport:
-            sport = ' port-group %d-%d' % (sport[0], sport[1])
-        else:
-            sport = ''
-        if dport:
-            dport = ' port-group %d-%d' % (dport[0], dport[1])
-        else:
-            dport = ''
-
-        return (' %s %s %s%s %s%s' % (action, proto, saddr, sport, daddr, dport)).rstrip()
+        Args:
+          addr: str or ipaddr, address
+        Returns:
+          An address string suitable for the ACL.
+        """
+        if addr.prefixlen == 0:
+            return f'{addr.parent_token}'
+        return f'net-group {addr.parent_token}'
 
 
 class Cisco(aclgenerator.ACLGenerator):
@@ -938,7 +858,7 @@ class Cisco(aclgenerator.ACLGenerator):
     _PROTO_INT = True
     _TERM_REMARK = True
 
-    def _BuildTokens(self):
+    def _BuildTokens(self) -> Tuple[Set[str], Dict[str, Set[str]]]:
         """Build supported tokens for platform.
 
         Returns:
@@ -965,7 +885,7 @@ class Cisco(aclgenerator.ACLGenerator):
         )
         return supported_tokens, supported_sub_tokens
 
-    def _TranslatePolicy(self, pol, exp_info):
+    def _TranslatePolicy(self, pol: Policy, exp_info: int) -> None:
         self.cisco_policies = []
         # a mixed filter outputs both ipv4 and ipv6 acls in the same output file
         good_filters = ['extended', 'standard', 'object-group', 'inet6', 'mixed', 'enable_dsmo']
@@ -1044,7 +964,7 @@ class Cisco(aclgenerator.ACLGenerator):
                             len(filter_options) > 2 and filter_options[2] == 'enable_dsmo'
                         )
                         new_terms.append(
-                            Term(
+                            ExtendedTerm(
                                 term,
                                 proto_int=self._PROTO_INT,
                                 enable_dsmo=enable_dsmo,
@@ -1055,12 +975,10 @@ class Cisco(aclgenerator.ACLGenerator):
                         )
                     elif next_filter == 'object-group':
                         obj_target.AddTerm(term)
-                        new_terms.append(
-                            self._GetObjectGroupTerm(term, filter_name, verbose=self.verbose)
-                        )
+                        new_terms.append(self._GetObjectGroupTerm(term, verbose=self.verbose))
                     elif next_filter == 'inet6':
                         new_terms.append(
-                            Term(
+                            ExtendedTerm(
                                 term,
                                 6,
                                 proto_int=self._PROTO_INT,
@@ -1076,11 +994,11 @@ class Cisco(aclgenerator.ACLGenerator):
                     (header, filter_name, [next_filter], new_terms, obj_target)
                 )
 
-    def _GetObjectGroupTerm(self, term, filter_name, verbose=True):
+    def _GetObjectGroupTerm(self, term: Term, verbose: bool = True) -> ObjectGroupTerm:
         """Returns an ObjectGroupTerm object."""
-        return ObjectGroupTerm(term, filter_name, verbose=verbose)
+        return ObjectGroupTerm(term, verbose=verbose)
 
-    def _AppendTargetByFilterType(self, filter_name, filter_type):
+    def _AppendTargetByFilterType(self, filter_name: str, filter_type: str) -> List[str]:
         """Takes in the filter name and type and appends headers.
 
         Args:
@@ -1115,7 +1033,9 @@ class Cisco(aclgenerator.ACLGenerator):
             )
         return target
 
-    def _RepositoryTagsHelper(self, target=None, filter_type='', filter_name=''):
+    def _RepositoryTagsHelper(
+        self, target: Optional[List[str]] = None, filter_type: str = '', filter_name: str = ''
+    ) -> List[str]:
         if target is None:
             target = []
         if filter_type == 'standard' and filter_name.isdigit():
@@ -1128,7 +1048,7 @@ class Cisco(aclgenerator.ACLGenerator):
             target.extend(aclgenerator.AddRepositoryTags(' remark ', date=False, revision=False))
         return target
 
-    def __str__(self):
+    def __str__(self) -> str:
         target_header = []
         target = []
         # add the p4 tags
