@@ -16,11 +16,10 @@
 
 """Juniper JCL generator."""
 
-import datetime
+
 from absl import logging
-from aerleon.lib import aclgenerator
-from aerleon.lib import nacaddr
-from aerleon.lib import summarizer
+
+from aerleon.lib import aclgenerator, nacaddr, summarizer
 
 
 # generic error class
@@ -211,15 +210,6 @@ class Term(aclgenerator.Term):
     #  eg, indent(8) + 'foo;' -> '%s%s;' % (indent(8), 'foo'). pyglint likes this
     #  more.
     def __str__(self):
-        # Verify platform specific terms. Skip whole term if platform does not
-        # match.
-        if self.term.platform:
-            if self._PLATFORM not in self.term.platform:
-                return ''
-        if self.term.platform_exclude:
-            if self._PLATFORM in self.term.platform_exclude:
-                return ''
-
         config = Config(indent=self._DEFAULT_INDENT)
         from_str = []
         # Don't render icmpv6 protocol terms under inet, or icmp under inet6
@@ -981,13 +971,7 @@ class Juniper(aclgenerator.ACLGenerator):
 
     def _TranslatePolicy(self, pol, exp_info):
         self.juniper_policies = []
-        current_date = datetime.datetime.utcnow().date()
-        exp_info_date = current_date + datetime.timedelta(weeks=exp_info)
-
         for header, terms in pol.filters:
-            if self._PLATFORM not in header.platforms:
-                continue
-
             filter_options = header.FilterOptions(self._PLATFORM)
             filter_name = header.FilterName(self._PLATFORM)
 
@@ -1064,21 +1048,6 @@ class Juniper(aclgenerator.ACLGenerator):
                     if not term:
                         continue
 
-                    if term.expiration:
-                        if term.expiration <= exp_info_date:
-                            logging.info(
-                                'INFO: Term %s in policy %s expires ' 'in less than two weeks.',
-                                term.name,
-                                filter_name,
-                            )
-                        if term.expiration <= current_date:
-                            logging.warning(
-                                'WARNING: Term %s in policy %s is expired and '
-                                'will not be rendered.',
-                                term.name,
-                                filter_name,
-                            )
-                            continue
                     if 'is-fragment' in term.option and filter_type == 'inet6':
                         raise JuniperFragmentInV6Error(
                             'The term %s uses "is-fragment" but ' 'is a v6 policy.' % term.name
