@@ -16,14 +16,12 @@
 
 """Nsxv generator."""
 
-import datetime
 import re
 import xml
 
 from absl import logging
-from aerleon.lib import aclgenerator
-from aerleon.lib import nacaddr
 
+from aerleon.lib import aclgenerator, nacaddr
 
 _ACTION_TABLE = {
     'accept': 'allow',
@@ -129,15 +127,6 @@ class Term(aclgenerator.Term):
           NsxvAclTermError: When unknown icmp-types are specified
 
         """
-        # Verify platform specific terms. Skip whole term if platform does not
-        # match.
-        if self.term.platform:
-            if 'nsxv' not in self.term.platform:
-                return ''
-        if self.term.platform_exclude:
-            if 'nsxv' in self.term.platform_exclude:
-                return ''
-
         ret_str = ['']
 
         # Don't render icmpv6 protocol terms under inet, or icmp under inet6
@@ -523,13 +512,7 @@ class Nsxv(aclgenerator.ACLGenerator):
 
     def _TranslatePolicy(self, pol, exp_info):
         self.nsxv_policies = []
-        current_date = datetime.datetime.utcnow().date()
-        exp_info_date = current_date + datetime.timedelta(weeks=exp_info)
-
         for header, terms in pol.filters:
-            if self._PLATFORM not in header.platforms:
-                continue
-
             filter_options = header.FilterOptions(self._PLATFORM)
             if len(filter_options) >= 2:
                 filter_name = filter_options[1]
@@ -548,21 +531,6 @@ class Nsxv(aclgenerator.ACLGenerator):
                     raise NsxvDuplicateTermError('There are multiple terms named: %s' % term.name)
                 term_names.add(term.name)
 
-                if term.expiration:
-                    if term.expiration <= exp_info_date:
-                        logging.info(
-                            'INFO: Term %s in policy %s expires ' 'in less than two weeks.',
-                            term.name,
-                            filter_name,
-                        )
-                    if term.expiration <= current_date:
-                        logging.warning(
-                            'WARNING: Term %s in policy %s is expired and '
-                            'will not be rendered.',
-                            term.name,
-                            filter_name,
-                        )
-                        continue
                 # Get the mapped action value
                 # If there is no mapped action value term is not rendered
                 mapped_action = _ACTION_TABLE.get(str(term.action[0]))
